/*
 * This file is part of WebGoat, an Open Web Application Security Project utility. For details, please see http://www.owasp.org/
 *
 * Copyright (c) 2002 - 2019 Bruce Mayhew
 *
 * This program is free software; you can redistribute it and/or modify it under the terms of the
 * GNU General Public License as published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without
 * even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with this program; if
 * not, write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * Getting Source ==============
 *
 * Source for this application is maintained at https://github.com/WebGoat/WebGoat, a repository for free software projects.
 */

package org.owasp.webgoat.lessons.webwolf_introduction;

import org.apache.commons.lang3.StringUtils;
import org.owasp.webgoat.container.assignments.AssignmentEndpoint;
import org.owasp.webgoat.container.assignments.AttackResult;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.client.RestClientException;
import org.springframework.web.client.RestTemplate;

/**
 * @author nbaars
 * @since 8/20/17.
 */
@RestController
public class MailAssignment extends AssignmentEndpoint {

    private final String webWolfURL;
    private RestTemplate restTemplate;

    public MailAssignment(RestTemplate restTemplate, @Value("${webwolf.mail.url}") String webWolfURL) {
        this.restTemplate = restTemplate;
        this.webWolfURL = webWolfURL;
    }

    @PostMapping("/WebWolf/mail/send")
    @ResponseBody
    public AttackResult sendEmail(@RequestParam String email) {
        String username = email.substring(0, email.indexOf("@"));
        if (username.equalsIgnoreCase(getWebSession().getUserName())) {
            Email mailEvent = Email.builder()
                    .recipient(username)
                    .title("Test messages from WebWolf")
                    .contents("This is a test message from WebWolf, your unique code is: " + StringUtils.reverse(username))
                    .sender("webgoat@owasp.org")
                    .build();
            try {
                restTemplate.postForEntity(webWolfURL, mailEvent, Object.class);
            } catch (RestClientException e ) {
                return informationMessage(this).feedback("webwolf.email_failed").output(e.getMessage()).build();
            }
            return informationMessage(this).feedback("webwolf.email_send").feedbackArgs(email).build();
        } else {
            return informationMessage(this).feedback("webwolf.email_mismatch").feedbackArgs(username).build();
        }
    }

    @PostMapping("/WebWolf/mail")
    @ResponseBody
    public AttackResult completed(@RequestParam String uniqueCode) {
        if (uniqueCode.equals(StringUtils.reverse(getWebSession().getUserName()))) {
            return success(this).build();
        } else {
            return failed(this).feedbackArgs("webwolf.code_incorrect").feedbackArgs(uniqueCode).build();
        }
    }
}
